// required for old g++ to compile PRId64 macros, see
// https://github.com/pytorch/pytorch/issues/3571
// for context
#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS
#endif

// an external backend might generate file within its code tree
// and check all the source files within the tree with clang-format.
// so, disable it since the backend might have a different config.
// clang-format off

// NOTE: This condition is true for all PyTorch internal libraries, it
//       just excludes external projects such as torch_xla which
//       re-use some of the PyTorch codegen machinery.
#if defined(CAFFE2_BUILD_MAIN_LIB)        || \
    defined(TORCH_CUDA_BUILD_MAIN_LIB)    || \
    defined(TORCH_HIP_BUILD_MAIN_LIB)     || \
    defined(TORCH_CUDA_CU_BUILD_MAIN_LIB) || \
    defined(TORCH_CUDA_CPP_BUILD_MAIN_LIB)
#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
#endif

// @generated by torchgen/gen.py from RegisterDispatchKey.cpp

#include <c10/core/TensorImpl.h>
#include <c10/core/Allocator.h>
#include <ATen/DeviceGuard.h>
#include <ATen/NamedTensorUtils.h>
#include <ATen/Utils.h>
#include <ATen/WrapDimUtils.h>
#include <ATen/Dispatch.h>
#include <c10/util/ExclusivelyOwned.h>
#include <c10/util/Half.h>
#include <c10/core/UndefinedTensorImpl.h>
#include <c10/util/Optional.h>
#include <ATen/Tensor.h>
#include <ATen/native/Resize.h>

#include <cstddef>
#include <functional>
#include <memory>
#include <utility>

#include <ATen/Config.h>
#include <ATen/core/op_registration/adaption.h>
#include <torch/library.h>


#include <ATen/ops/as_strided_native.h>
#include <ATen/ops/empty.h>
#include <ATen/ops/empty_strided.h>
#include <ATen/ops/_copy_from_and_resize.h>
#include <ATen/ops/_copy_from.h>
#include <ATen/ops/_nnz_native.h>
#include <ATen/ops/add_native.h>
#include <ATen/ops/ccol_indices_native.h>
#include <ATen/ops/col_indices_native.h>
#include <ATen/ops/crow_indices_native.h>
#include <ATen/ops/dense_dim_native.h>
#include <ATen/ops/empty_like_native.h>
#include <ATen/ops/empty_native.h>
#include <ATen/ops/row_indices_native.h>
#include <ATen/ops/sparse_dim_native.h>
#include <ATen/ops/sum_native.h>
#include <ATen/ops/values_native.h>
#include <ATen/ops/zero_native.h>

// See template file RegisterDispatchDefinitions.ini
namespace at {
// NB: TORCH_LIBRARY_IMPL must be in an anonymous namespace to avoid
// ambiguity with conflicting identifiers that may have been defined in
// at namespace already.
namespace {
void resize_out(const Tensor &out, IntArrayRef sizes, IntArrayRef strides, const TensorOptions &options) {
  TORCH_CHECK(options.dtype() == out.dtype(),
      "Expected out tensor to have dtype ", options.dtype(), ", but got ", out.dtype(), " instead");
  TORCH_CHECK(options.device() == out.device(),
      "Expected out tensor to have device ", options.device(), ", but got ", out.device(), " instead");
  const bool resized = at::native::resize_output(out, sizes);
  // Only restride if a resize occurred; otherwise we ignore the (advisory)
  // strides from the meta function and directly use the output tensor's
  // preexisting strides
  if (resized) {
    if (!strides.empty()) {
      TORCH_INTERNAL_ASSERT(!options.memory_format_opt().has_value());
      // TODO: avoid the redispatch here
      out.as_strided_(sizes, strides);
    } else if (options.memory_format_opt().has_value()) {
      out.unsafeGetTensorImpl()->empty_tensor_restride(*options.memory_format_opt());
    }
  }
}
void check_inplace(const Tensor &self, IntArrayRef sizes, const TensorOptions &options) {
  // These checks are needed on those operators that:
  //   1) don't use 'TensorIterator' (e.g. 'addmm' and 'baddbmm')
  //   2) have particular typing rules (e.g. 'cumsum' and 'cumprod')
  // For other operators (e.g. 'add'), 'TensorIterator' already checks
  // these things separately.
  TORCH_CHECK(options.dtype() == self.dtype(),
      "Bad in-place call: ",
      "input tensor dtype ", self.dtype(), " and output tensor dtype ", options.dtype(), " should match");
  TORCH_CHECK(options.device() == self.device(),
      "Bad in-place call: ",
      "input tensor device ", self.device(), " and output tensor device ", options.device(), " should match");
  TORCH_CHECK(sizes == self.sizes(),
      "Bad in-place call: ",
      "input tensor size ", self.sizes(), " and output tensor size ", sizes, " should match");
}
namespace {
at::Tensor wrapper_SparseCsrMeta_Tensor_add(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
    // No device check
  // DeviceGuard omitted
  return at::native::add_sparse_csr(self, other, alpha);
}
} // anonymous namespace
namespace {
at::Tensor & wrapper_SparseCsrMeta_out_add_out(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha, at::Tensor & out) {
    // No device check
  // DeviceGuard omitted
  return at::native::add_out_sparse_compressed_cpu(self, other, alpha, out);
}
} // anonymous namespace
namespace {
at::Tensor & wrapper_SparseCsrMeta_Tensor_add_(at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
    // No device check
  // DeviceGuard omitted
  return at::native::add_sparse_csr_(self, other, alpha);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta_memory_format_empty(c10::SymIntArrayRef size, ::std::optional<at::ScalarType> dtype, ::std::optional<at::Layout> layout, ::std::optional<at::Device> device, ::std::optional<bool> pin_memory, ::std::optional<at::MemoryFormat> memory_format) {
    // No device check
  // DeviceGuard omitted
  return at::native::empty_sparse_compressed(C10_AS_INTARRAYREF_SLOW(size), dtype, layout, device, pin_memory, memory_format);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__empty_like(const at::Tensor & self, ::std::optional<at::ScalarType> dtype, ::std::optional<at::Layout> layout, ::std::optional<at::Device> device, ::std::optional<bool> pin_memory, ::std::optional<at::MemoryFormat> memory_format) {
    // No device check
  // DeviceGuard omitted
  return at::native::empty_like_sparse_csr(self, dtype, layout, device, pin_memory, memory_format);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__sum(const at::Tensor & self, ::std::optional<at::ScalarType> dtype) {
    // No device check
  // DeviceGuard omitted
  return at::native::sum_csr(self, dtype);
}
} // anonymous namespace
namespace {
at::Tensor & wrapper_SparseCsrMeta__zero_(at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::zero_sparse_csr_(self);
}
} // anonymous namespace
namespace {
int64_t wrapper_SparseCsrMeta__sparse_dim(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::sparse_dim_sparse_csr(self);
}
} // anonymous namespace
namespace {
int64_t wrapper_SparseCsrMeta__dense_dim(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::dense_dim_sparse_csr(self);
}
} // anonymous namespace
namespace {
int64_t wrapper_SparseCsrMeta___nnz(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::_nnz_sparse_csr(self);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__values(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::values_sparse_csr(self);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__crow_indices(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::crow_indices_sparse_csr(self);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__col_indices(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::col_indices_sparse_csr(self);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__ccol_indices(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::ccol_indices_sparse_csr(self);
}
} // anonymous namespace
namespace {
at::Tensor wrapper_SparseCsrMeta__row_indices(const at::Tensor & self) {
    // No device check
  // DeviceGuard omitted
  return at::native::row_indices_sparse_csr(self);
}
} // anonymous namespace
TORCH_LIBRARY_IMPL(aten, SparseCsrMeta, m) {
    m.impl("add.Tensor",
TORCH_FN(wrapper_SparseCsrMeta_Tensor_add));
m.impl("add.out",
TORCH_FN(wrapper_SparseCsrMeta_out_add_out));
m.impl("add_.Tensor",
TORCH_FN(wrapper_SparseCsrMeta_Tensor_add_));
m.impl("empty.memory_format",
TORCH_FN(wrapper_SparseCsrMeta_memory_format_empty));
m.impl("empty_like",
TORCH_FN(wrapper_SparseCsrMeta__empty_like));
m.impl("sum",
TORCH_FN(wrapper_SparseCsrMeta__sum));
m.impl("zero_",
TORCH_FN(wrapper_SparseCsrMeta__zero_));
m.impl("sparse_dim",
TORCH_FN(wrapper_SparseCsrMeta__sparse_dim));
m.impl("dense_dim",
TORCH_FN(wrapper_SparseCsrMeta__dense_dim));
m.impl("_nnz",
TORCH_FN(wrapper_SparseCsrMeta___nnz));
m.impl("values",
TORCH_FN(wrapper_SparseCsrMeta__values));
m.impl("crow_indices",
TORCH_FN(wrapper_SparseCsrMeta__crow_indices));
m.impl("col_indices",
TORCH_FN(wrapper_SparseCsrMeta__col_indices));
m.impl("ccol_indices",
TORCH_FN(wrapper_SparseCsrMeta__ccol_indices));
m.impl("row_indices",
TORCH_FN(wrapper_SparseCsrMeta__row_indices));
};
} // anonymous namespace
namespace sparsecsrmeta {
at::Tensor add(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_SparseCsrMeta_Tensor_add(self, other, alpha);
}
at::Tensor & add_out(at::Tensor & out, const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_SparseCsrMeta_out_add_out(self, other, alpha, out);
}
at::Tensor & add_outf(const at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha, at::Tensor & out) {
return wrapper_SparseCsrMeta_out_add_out(self, other, alpha, out);
}
at::Tensor & add_(at::Tensor & self, const at::Tensor & other, const at::Scalar & alpha) {
return wrapper_SparseCsrMeta_Tensor_add_(self, other, alpha);
}
at::Tensor empty(at::IntArrayRef size, at::TensorOptions options, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta_memory_format_empty(c10::fromIntArrayRefSlow(size), c10::optTypeMetaToScalarType(options.dtype_opt()), options.layout_opt(), options.device_opt(), options.pinned_memory_opt(), c10::impl::check_tensor_options_and_extract_memory_format(options, memory_format));
}
at::Tensor empty(at::IntArrayRef size, ::std::optional<at::ScalarType> dtype, ::std::optional<at::Layout> layout, ::std::optional<at::Device> device, ::std::optional<bool> pin_memory, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta_memory_format_empty(c10::fromIntArrayRefSlow(size), dtype, layout, device, pin_memory, memory_format);
}
at::Tensor empty_symint(c10::SymIntArrayRef size, at::TensorOptions options, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta_memory_format_empty(size, c10::optTypeMetaToScalarType(options.dtype_opt()), options.layout_opt(), options.device_opt(), options.pinned_memory_opt(), c10::impl::check_tensor_options_and_extract_memory_format(options, memory_format));
}
at::Tensor empty_symint(c10::SymIntArrayRef size, ::std::optional<at::ScalarType> dtype, ::std::optional<at::Layout> layout, ::std::optional<at::Device> device, ::std::optional<bool> pin_memory, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta_memory_format_empty(size, dtype, layout, device, pin_memory, memory_format);
}
at::Tensor empty_like(const at::Tensor & self, at::TensorOptions options, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta__empty_like(self, c10::optTypeMetaToScalarType(options.dtype_opt()), options.layout_opt(), options.device_opt(), options.pinned_memory_opt(), c10::impl::check_tensor_options_and_extract_memory_format(options, memory_format));
}
at::Tensor empty_like(const at::Tensor & self, ::std::optional<at::ScalarType> dtype, ::std::optional<at::Layout> layout, ::std::optional<at::Device> device, ::std::optional<bool> pin_memory, ::std::optional<at::MemoryFormat> memory_format) {
return wrapper_SparseCsrMeta__empty_like(self, dtype, layout, device, pin_memory, memory_format);
}
at::Tensor sum(const at::Tensor & self, ::std::optional<at::ScalarType> dtype) {
return wrapper_SparseCsrMeta__sum(self, dtype);
}
at::Tensor & zero_(at::Tensor & self) {
return wrapper_SparseCsrMeta__zero_(self);
}
int64_t sparse_dim(const at::Tensor & self) {
return wrapper_SparseCsrMeta__sparse_dim(self);
}
int64_t dense_dim(const at::Tensor & self) {
return wrapper_SparseCsrMeta__dense_dim(self);
}
int64_t _nnz(const at::Tensor & self) {
return wrapper_SparseCsrMeta___nnz(self);
}
at::Tensor values(const at::Tensor & self) {
return wrapper_SparseCsrMeta__values(self);
}
at::Tensor crow_indices(const at::Tensor & self) {
return wrapper_SparseCsrMeta__crow_indices(self);
}
at::Tensor col_indices(const at::Tensor & self) {
return wrapper_SparseCsrMeta__col_indices(self);
}
at::Tensor ccol_indices(const at::Tensor & self) {
return wrapper_SparseCsrMeta__ccol_indices(self);
}
at::Tensor row_indices(const at::Tensor & self) {
return wrapper_SparseCsrMeta__row_indices(self);
}
} // namespace sparsecsrmeta
} // namespace at
